import gym
from stable_baselines3 import A2C
import wandb
from wandb.integration.sb3 import WandbCallback
from stable_baselines3.common.env_util import make_vec_env

# start a new wandb run to track this script
config = {
    "policy_type": "MlpPolicy",
    "total_timesteps": 25000,
    "env_name": "CartPole-v1",
}

run = wandb.init(
    project="cartpole",
    config=config,
    sync_tensorboard=True,
    monitor_gym=True,
    save_code=True, 
)
env = make_vec_env("CartPole-v1", n_envs=4)

# Train the model
model = A2C(config["policy_type"], env, verbose=1, tensorboard_log=f"runs/{run.id}")
model.learn(
    total_timesteps=config["total_timesteps"],
    callback=WandbCallback()
)

run.finish()

if False:
    from huggingface_sb3 import push_to_hub
    from huggingface_hub import login

    login(token="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")
    push_to_hub(
        repo_id="JulesCoulon/A2C_CartPole",
        filename="a2c_cartpole.zip",
        commit_message="Added A2C model for CartPole with Stable Baselines3",
    )