from tqdm import tqdm
import torch
import gym
from torch.distributions import Categorical
import torch.nn as nn

class SimpleNN(nn.Module):
    def __init__(self):
        super(SimpleNN, self).__init__()
        
        # Première couche entièrement connectée avec 128 unités et activation ReLU
        self.fc1 = nn.Linear(4, 128) # sortie de l'observation
        self.relu = nn.ReLU()
        self.dropout = nn.Dropout(0.25)
        
        # Deuxième couche entièrement connectée suivie d'une activation Softmax
        self.fc2 = nn.Linear(128, 2) # 2 action space
        self.softmax = nn.Softmax(dim=0)
    
    def forward(self, x):
        x = self.fc1(x)
        x = self.relu(x)
        x = self.dropout(x)
        x = self.fc2(x)
        x = self.softmax(x)
        return x


model = SimpleNN()
model.load_state_dict(torch.load("reinforce_cartpole.pth"))
model.eval()
# Create the environment
env = gym.make("CartPole-v1", render_mode=None)

achievement = 0

for ep in tqdm(range(100)):    
    observation = env.reset()[0]
    terminated = False
    for id in range(500):
        prob = model(torch.tensor(observation))
        # Choose the action with the highest probability
        action = torch.argmax(prob)
        observation, reward, terminated, truncated, info = env.step(action.item())
        if terminated:
            print("Episode terminated at step ", id)
            break
    if not terminated: 
        print("Episode terminated at step ", id)
        achievement += 1
    
print("Achievement rate : ", achievement, "%")
env.close()