import gym
import torch
import torch.nn as nn
import torch.optim as optim
from torch.distributions import Categorical
import torch.nn.functional as F
import matplotlib.pyplot as plt


env = gym.make("CartPole-v1", render_mode=None)
# Définition du modèle
class SimpleNN(nn.Module):
    def __init__(self):
        super(SimpleNN, self).__init__()
        
        # Première couche entièrement connectée avec 128 unités et activation ReLU
        self.fc1 = nn.Linear(4, 128) # sortie de l'observation
        self.relu = nn.ReLU()
        self.dropout = nn.Dropout(0.25)
        
        # Deuxième couche entièrement connectée suivie d'une activation Softmax
        self.fc2 = nn.Linear(128, 2) # 2 action space
        self.softmax = nn.Softmax(dim=0)
    
    def forward(self, x):
        x = self.fc1(x)
        x = self.relu(x)
        x = self.dropout(x)
        x = self.fc2(x)
        x = self.softmax(x)
        return x

# Instancier le modèle
model = SimpleNN()

gamma = 0.99
# Définir l'optimiseur et la fonction de perte
optimizer = optim.Adam(model.parameters(), lr=5*10**-3)
criterion = nn.CrossEntropyLoss()  # La CrossEntropyLoss inclut Softmax, donc pas besoin de le redéfinir

# Affichage du modèle
print(model)

nb_episode = 500
max_episode_steps = 500
total_rewards = []
total_loss = []
for ep in range(nb_episode):
    observation = env.reset()[0]
    buffer = torch.zeros(max_episode_steps + 1)
    probs = torch.zeros(max_episode_steps + 1)
    done = False
    id = 0
    terminated = False
    while not(terminated) and id < max_episode_steps:
        prob = model(torch.tensor(observation))
        m = Categorical(prob)
        action = m.sample()
        probs[id] = prob[action]
        observation, reward, terminated, truncated, info = env.step(action.item())
        for i in range(id+1):
            buffer[i] += reward * gamma**(i-1)
        id += 1
    # env.render()
    total_rewards.append(id)    
    probs = probs[:id]
    buffer = buffer[:id]
    F.normalize(buffer, dim=0)
    
    # Compute loss
    logs = torch.log10(probs)
    loss = - torch.sum(torch.mul(logs, buffer))
    total_loss.append(loss)
    
    # Print progress
    if (ep + 1) % 50 == 0:
        print("Épisode : {} / {}".format(ep + 1, nb_episode))
        print("Reward : ", id)
        print("Loss : ", round(loss.item(), 2))


    # Perform gradient descent to update neural network
    optimizer.zero_grad()
    loss.backward()
    optimizer.step()


# Plot the evolution of learning : reward and loss
# Reward
plt.plot(total_rewards)
plt.xlabel('Épisode')
plt.ylabel('Reward')
plt.title("Évolution du reward en fonction de l'épisode")
plt.show()

# Loss
total_losses = [loss.item() for loss in total_loss]
plt.plot(total_losses)
plt.xlabel('Épisode')
plt.ylabel('Loss')
plt.title("Évolution du loss en fonction de l'épisode")
plt.show()
